# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

class Backend:
    ...

def get_backend(name: str) -> Backend:
    ...
def sniff() -> str:
    ...
def get_default_backend() -> Backend:
    ...
def set_default_backend(name: str) -> Backend:
    ...
